﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace XNAGeometryObject
{
    public class Box
    {
        public VertexPositionNormalTexture[] Vertices { get { return vertices; } }
        private VertexPositionNormalTexture[] vertices;

        public Box( float size )
        {
            createVertices( size );
        }

        /// <summary>
        /// Create the vertices for the box
        /// </summary>
        private void createVertices( float size )
        {
            vertices = new VertexPositionNormalTexture[ 36 ];

            // Define the vertcies of our box
            Vector3 topLeftFront = new Vector3(-1.0f, 1.0f, 1.0f);
            Vector3 topRightFront = new Vector3(1.0f, 1.0f, 1.0f);
            Vector3 bottomRightFront = new Vector3(1.0f, -1.0f, 1.0f);
            Vector3 bottomLeftFront = new Vector3(-1.0f, -1.0f, 1.0f);
            Vector3 topLeftBack = new Vector3(-1.0f, 1.0f, -1.0f);
            Vector3 topRightBack = new Vector3(1.0f, 1.0f, -1.0f);
            Vector3 bottomRightBack = new Vector3(1.0f, -1.0f, -1.0f);
            Vector3 bottomLeftBack = new Vector3(-1.0f, -1.0f, -1.0f);

            // Define our normals so we can use directional lighting
            Vector3 frontNormal = new Vector3(0.0f, 0.0f, 1.0f);
            Vector3 backNormal = new Vector3(0.0f, 0.0f, -1.0f);
            Vector3 topNormal = new Vector3(0.0f, 1.0f, 0.0f);
            Vector3 bottomNormal = new Vector3(0.0f, -1.0f, 0.0f);
            Vector3 rightNormal = new Vector3(1.0f, 0.0f, 0.0f);
            Vector3 leftNormal = new Vector3(-1.0f, 0.0f, 0.0f);

            // Define our texture coordinates
            Vector2 textureBottomLeft = new Vector2(0.0f, 1.0f);
            Vector2 textureTopLeft = new Vector2(0.0f, 0.0f);
            Vector2 textureTopRight = new Vector2(1.0f, 0.0f);
            Vector2 textureBottomRight = new Vector2(1.0f, 1.0f);

            // Front face.
            vertices[0] = new VertexPositionNormalTexture(
                bottomLeftFront, frontNormal, textureBottomLeft);
            vertices[1] = new VertexPositionNormalTexture(
                topLeftFront, frontNormal, textureTopLeft);
            vertices[2] = new VertexPositionNormalTexture(
                topRightFront, frontNormal, textureTopRight);
            vertices[3] = new VertexPositionNormalTexture(
                bottomLeftFront, frontNormal, textureBottomLeft);
            vertices[4] = new VertexPositionNormalTexture(
                topRightFront, frontNormal, textureTopRight);
            vertices[5] = new VertexPositionNormalTexture(
                bottomRightFront, frontNormal, textureBottomRight);

            // Back face.
            vertices[6] = new VertexPositionNormalTexture(
                bottomRightBack, backNormal, textureBottomLeft);               
            vertices[7] = new VertexPositionNormalTexture(
                topRightBack, backNormal, textureTopLeft);
            vertices[8] = new VertexPositionNormalTexture(
                topLeftBack, backNormal, textureTopRight);
            vertices[9] = new VertexPositionNormalTexture(
                bottomRightBack, backNormal, textureBottomLeft);
            vertices[10] = new VertexPositionNormalTexture(
                topLeftBack, backNormal, textureTopRight);
            vertices[11] = new VertexPositionNormalTexture(
                bottomLeftBack, backNormal, textureBottomRight);

            // Top face.
            vertices[12] = new VertexPositionNormalTexture(
                topLeftFront, topNormal, textureBottomLeft); 
            vertices[13] = new VertexPositionNormalTexture(
                topLeftBack, topNormal, textureTopLeft);
            vertices[14] = new VertexPositionNormalTexture(
                topRightBack, topNormal, textureTopRight);
            vertices[15] = new VertexPositionNormalTexture(
                topLeftFront, topNormal, textureBottomLeft);
            vertices[16] = new VertexPositionNormalTexture(
                topRightBack, topNormal, textureTopRight);
            vertices[17] = new VertexPositionNormalTexture(
                topRightFront, topNormal, textureBottomRight);

            // Bottom face.
            vertices[18] = new VertexPositionNormalTexture(
                bottomLeftBack, bottomNormal, textureBottomLeft);
            vertices[19] = new VertexPositionNormalTexture(
                bottomLeftFront, bottomNormal, textureTopLeft);
            vertices[20] = new VertexPositionNormalTexture(
                bottomRightFront, bottomNormal, textureTopRight);
            vertices[21] = new VertexPositionNormalTexture(
                bottomLeftBack, bottomNormal, textureBottomLeft);
            vertices[22] = new VertexPositionNormalTexture(
                bottomRightFront, bottomNormal, textureTopRight);           
            vertices[23] = new VertexPositionNormalTexture(
                bottomRightBack, bottomNormal, textureBottomRight);

            // Right face.
            vertices[24] = new VertexPositionNormalTexture(
                bottomRightFront, rightNormal, textureBottomLeft);
            vertices[25] = new VertexPositionNormalTexture(
                topRightFront, rightNormal, textureTopLeft);
            vertices[26] = new VertexPositionNormalTexture(
                topRightBack, rightNormal, textureTopRight);
            vertices[27] = new VertexPositionNormalTexture(
                bottomRightFront, rightNormal, textureBottomLeft);
            vertices[28] = new VertexPositionNormalTexture(
                topRightBack, rightNormal, textureTopRight);
            vertices[29] = new VertexPositionNormalTexture(
                bottomRightBack, rightNormal, textureBottomRight);

            // Left face.
            vertices[30] = new VertexPositionNormalTexture(
                bottomLeftBack, leftNormal, textureBottomLeft);
            vertices[31] = new VertexPositionNormalTexture(
                topLeftBack, leftNormal, textureTopLeft);
            vertices[32] = new VertexPositionNormalTexture(
                topLeftFront, leftNormal, textureTopRight);
            vertices[33] = new VertexPositionNormalTexture(
                bottomLeftBack, leftNormal, textureBottomLeft);
            vertices[34] = new VertexPositionNormalTexture(
                topLeftFront, leftNormal, textureTopRight);
            vertices[35] = new VertexPositionNormalTexture(
                bottomLeftFront, leftNormal, textureBottomRight);

        }

    }
}
