float4x4 World;
float4x4 View;
float4x4 Projection;

// light
float3 EyePosition;
float3 LightPosition;
float4 AmbientLightColor;
float4 DiffuseLightColor;
float4 SpecularLightColor;
float SpecularPower;
float SpecularIntensity;

// texture
texture DiffuseTexture;
sampler2D DiffuseSampler = sampler_state {
    Texture = <DiffuseTexture>;
    MinFilter = Anisotropic;
    MipFilter = LINEAR;
    MagFilter = LINEAR;
	MaxAnisotropy = 8; 
    AddressU = WRAP;
    AddressV = MIRROR;
};

struct VertexShaderInput
{
    float4 Position : POSITION0;
    float3 DiffuseTexCoord : TEXCOORD0;
};

struct VertexShaderOutput
{
    float4 Position : POSITION;	// transformed position
    float3 DiffuseTexCoord : TEXCOORD0;		// texture
};

VertexShaderOutput VertexShaderFunction(VertexShaderInput input)
{
    VertexShaderOutput output;

    float4 worldPosition = mul(input.Position, World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);
	output.DiffuseTexCoord = input.DiffuseTexCoord;

    return output;
}

float4 PixelShaderFunction(VertexShaderOutput input) : COLOR0
{
    // TODO: add your pixel shader code here.

    return tex2D(DiffuseSampler, input.DiffuseTexCoord);
}

technique DefaultTextureTechnique
{
    pass Pass1
    {
        // TODO: set renderstates here.

        VertexShader = compile vs_2_0 VertexShaderFunction();
        PixelShader = compile ps_2_0 PixelShaderFunction();
    }
}

//////////////////////////////////////////////////////
// per-pixel lighting 
struct PerPixelLightingVertexShaderInput 
{
    float4 Position : POSITION;
    float3 DiffuseTexCoord : TEXCOORD0;
    float3 Normal : TEXCOORD1;
};

struct PerPixelLightingVertexShaderOutput 
{
    float4 Position : POSITION;
    float3 DiffuseTexCoord : TEXCOORD0;
    float3 WorldNormal : TEXCOORD1;
    float3 WorldPosition : TEXCOORD2;
};

struct PerPixelLightingPixelShaderInput
{
	float3 DiffuseTexCoord : TEXCOORD0;
	float3 WorldNormal: TEXCOORD1;
    float3 WorldPosition : TEXCOORD2;
};

/*
PerPixelLightingVertexShaderOutput PerPixelLightingVertexShaderFunction(PerPixelLightingVertexShaderInput input)
{
    PerPixelLightingVertexShaderOutput output;

    float4 worldPosition = mul(input.Position, World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);
	output.DiffuseTexCoord = input.DiffuseTexCoord;
	output.Normal = mul(input.Normal, World);

    return output;
}
*/

PerPixelLightingVertexShaderOutput PerPixelLightingVertexShaderFunction(
	float3 Position : POSITION,
	float3 TexCoord : TEXCOORD0,
    float3 Normal : NORMAL)
{
    PerPixelLightingVertexShaderOutput output;

    float4 worldPosition = mul(float4(Position, 1.0), World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);
	output.DiffuseTexCoord = TexCoord;
	output.WorldNormal = mul(Normal, World);
	output.WorldPosition = worldPosition / worldPosition.w;

    return output;
}

float4 PerPixelLightingPixelShaderFunction(PerPixelLightingPixelShaderInput input) : COLOR
{
    // TODO: add your pixel shader code here.
	float3 directionToLight = normalize(LightPosition - input.WorldPosition);
	float diffuseIntensity =  saturate(dot(directionToLight, input.WorldNormal));
	float4 diffuse = DiffuseLightColor * diffuseIntensity;

	//calculate Phong components per-pixel
    float3 reflectionVector = normalize(reflect(-directionToLight, input.WorldNormal));
    float3 directionToEye = normalize(EyePosition - input.WorldPosition);
     
    //calculate specular component
    float4 specular = SpecularLightColor * SpecularIntensity * 
                      pow(saturate(dot(reflectionVector, directionToEye)), SpecularPower);

    //all color components are summed in the pixel shader
	float4 lightColor = specular + diffuse + AmbientLightColor;
	lightColor.a = 1.0;
	
    return lightColor * tex2D(DiffuseSampler, input.DiffuseTexCoord);
}

technique PerPixelDiffuseLightingTextureTechnique
{
    pass Pass1
    {
        // TODO: set renderstates here.

        VertexShader = compile vs_2_0 PerPixelLightingVertexShaderFunction();
        PixelShader = compile ps_2_0 PerPixelLightingPixelShaderFunction();
    }
}
