using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using XNAGeometryObject;

namespace XNATextureBoxEffect
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // draw cube
        Box box;
        Texture2D texture;

        // effect
        Effect effect;
        Matrix projection;
        Matrix view;
        Matrix world;
        string techniqueName;

        // input
        GamePadState currentGamePadState;
        GamePadState prevGamePadState;
        KeyboardState currentKeyboardState;
        KeyboardState prevKeyboardState;
        bool animationMode = false;
        float angle = 0.0f;
        float zoom = 1.0f;

        // ambient, diffuse, specular light color
        Vector4 ambient = new Vector4(0.8f, 0.8f, 0.8f, 1.0f);
        Vector4 diffuse = new Vector4(1.0f, 1.0f, 1.0f, 1.0f);
        Vector4 specular = new Vector4(1.0f, 0.0f, 0.0f, 1.0f);
        Vector3 lightPosition = new Vector3(30.0f, 0.0f, 0.0f);
        Vector3 eyePosition = new Vector3(0.0f, 0.0f, 5.0f);  // same as LookAt

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            box = new Box(2);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            InitializeTransform();
            InitializeEffect();

            base.Initialize();
        }

        /// <summary>
        /// Initializes the transforms used by the game.
        /// </summary>
        private void InitializeTransform()
        {
            // world matrix
            world = Matrix.Identity;

            // view matrix
            view = Matrix.CreateLookAt(new Vector3(0, 0, 5.0f), Vector3.Zero, Vector3.Up);

            // projection matrix
            float aspectRatio = (float)GraphicsDevice.Viewport.Width / (float)GraphicsDevice.Viewport.Height;
            projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4, aspectRatio, 1.0f, 1000.0f);
        }

        /// <summary>
        /// Initializes the effect (loading, parameter setting, and technique selection)
        /// used by the game.
        /// </summary>
        private void InitializeEffect()
        {
            // Initialize Effect
            effect = Content.Load<Effect>("TextureEffect");

            effect.Parameters["World"].SetValue(world);
            effect.Parameters["View"].SetValue(view);
            effect.Parameters["Projection"].SetValue(projection);

            effect.Parameters["AmbientLightColor"].SetValue(ambient);
            effect.Parameters["DiffuseLightColor"].SetValue(diffuse);
            effect.Parameters["SpecularLightColor"].SetValue(specular);
            effect.Parameters["SpecularPower"].SetValue(16);
            effect.Parameters["SpecularIntensity"].SetValue(1);

            effect.Parameters["LightPosition"].SetValue(lightPosition);
            effect.Parameters["EyePosition"].SetValue(eyePosition);

            texture = Content.Load<Texture2D>("crate");
            effect.Parameters["DiffuseTexture"].SetValue(texture);

            techniqueName = "PerPixelDiffuseLightingTextureTechnique";
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            HandleInput();

            base.Update(gameTime);
        }

        /// <summary>
        /// Determines which primitive to draw based on input from the keyboard
        /// or game pad.
        /// </summary>
        protected void HandleInput()
        {
            CheckGamePadInput();
            CheckKeyboardInput();
        }

        private void CheckGamePadInput()
        {
            prevGamePadState = currentGamePadState;
            currentGamePadState = GamePad.GetState(PlayerIndex.One);
        }

        private void CheckKeyboardInput()
        {
            prevKeyboardState = currentKeyboardState;
            currentKeyboardState = Keyboard.GetState(PlayerIndex.One);

            if (currentKeyboardState.IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }
            else if (currentKeyboardState.IsKeyDown(Keys.X)
                && !prevKeyboardState.IsKeyDown(Keys.X))
            {
                angle += 5.0f;
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Y)
                && !prevKeyboardState.IsKeyDown(Keys.Y))
            {
                zoom -= 0.5f;
            }
            else if (currentKeyboardState.IsKeyDown(Keys.Space)
                && !prevKeyboardState.IsKeyDown(Keys.Space))
            {
                animationMode = !animationMode;
            }
            else if (currentKeyboardState.IsKeyDown(Keys.E)
                && !prevKeyboardState.IsKeyDown(Keys.E))
            {
                if (techniqueName == "DefaultTextureTechnique")
                    techniqueName = "PerPixelDiffuseLightingTextureTechnique";
                else
                    techniqueName = "DefaultTextureTechnique";
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            if (animationMode) angle += 0.8f;
            world = Matrix.CreateFromYawPitchRoll(MathHelper.ToRadians(angle), 45.0f, 0.0f) * Matrix.CreateTranslation(0, 0, zoom);
            effect.Parameters["World"].SetValue(world);

            // technique
            effect.CurrentTechnique = effect.Techniques[techniqueName];

            // effect
            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Apply();
                // Draw the box
                GraphicsDevice.DrawUserPrimitives<VertexPositionNormalTexture>(PrimitiveType.TriangleList,
                    box.Vertices, 0, box.Vertices.Length / 3);
            }

            base.Draw(gameTime);
        }
    }
}
